<?php

namespace App\Service;

use App\Exception\Exception;

class OrderService
{

    private $orderFactory;
    private $util;

    public function __construct(
        \App\Factory\OrderFactory $orderFactory,
        \App\Util\Util $util
    ) {
        $this->orderFactory = $orderFactory;
        $this->util = $util;
    }

    public function getComanda($nrcomanda)
    {
        $orderData = $this->orderFactory->getComanda($nrcomanda);
        if (empty($orderData))
            return null;

        return $orderData;
    }

    public function getTable($nrMesa)
    {
        $mesa = $this->orderFactory->getTable($nrMesa);
        if (empty($mesa))
            throw new \Exception("Mesa {$nrMesa} não cadastrada ou inválida");

        return $mesa;
    }

    public function updateTableOrder($nrMesa, $nrVendarest)
    {

        return $this->orderFactory->updateTableOrder($nrMesa, $nrVendarest);
    }

    public function updatePeopleQuantity($qtdPeople, $nrVendarest)
    {
        $this->orderFactory->updatePeopleQuantity($qtdPeople, $nrVendarest);
    }

    public function getTableIsOpen($nrMesa)
    {
        try {
            $orderData = $this->orderFactory->getTableIsOpen($nrMesa);
            if (empty($orderData))
                return null;

            return $orderData;
        } catch (\Exception $e) {
            throw new \Exception("Erro na consulta da mesa. $e");
        }
    }

    public function updateStatusTable($cdFilial, $nrMesa, $cdLojaa)
    {
        try {

            return $this->orderFactory->updateStatusTable($nrMesa);
        } catch (\Exception $e) {
            throw new \Exception("Erro na alteração de status da mesa. $e");
        }
    }

    public function openConsummation($cdFilial, $cdLoja, $generalParams, $dscomanda, $nrMesa, $orderType, $IDSTCOMANDA, $qtdPeople)
    {

        try {
            $vendarestSeq = $this->util->geraCodigo("VENDAREST$cdFilial", 1, 10);
            $comandavenSeq = $this->util->geraCodigo("COMANDAVEN$cdFilial", 1, 10);

            if (empty($dscomanda))
                $dscomanda = "PKR_$comandavenSeq";

            $this->orderFactory->insertVendarest($cdFilial, $cdLoja, $generalParams, $vendarestSeq, $nrMesa, $qtdPeople);
            $this->orderFactory->insertComandaven($cdFilial, $cdLoja, $generalParams, $vendarestSeq, $comandavenSeq, $dscomanda, $orderType, $IDSTCOMANDA);

            $return = [
                'NRVENDAREST' => $vendarestSeq,
                'NRCOMANDA'   => $comandavenSeq,
                'DSCOMANDA'   => $dscomanda
            ];

            return $return;
        } catch (Exception $e) {
            return ("Falha na abertura de Mesa/Comanda: " . $e);
        }
    }

    public function checkTableStatus($cdFilial, $nrMesa, $cdLoja)
    {
        try {

            $statusTable = $this->orderFactory->tableStatus($cdFilial, $nrMesa, $cdLoja);
            if (empty($statusTable))
                throw new \Exception('Mesa não encontrada');

            $listStatus = [
                'O' => [
                    'statusId' => 'A',
                    'status'  => 'Mesa Aberta'
                ],
                'D' => [
                    'statusId' => 'L',
                    'status'  => 'Mesa Livre'
                ],
                'S' => [
                    'statusId' => 'F',
                    'status'  => 'Conta Fechada'
                ],
                'R' => [
                    'statusId' => 'R',
                    'status'  => 'Mesa em Recebimento'
                ],
            ];
            $Status = isset($listStatus[$statusTable['IDSTMESAAUX']]) ? $listStatus[$statusTable['IDSTMESAAUX']] : null;

            return $Status;
        } catch (Exception $e) {
            throw new \Exception("Erro ao consultar status da mesa. $e");
        }
    }

    public function getIdcontroprod($cdLoja, $cdFilial)
    {
        return $this->orderFactory->getIdcontroprod($cdLoja, $cdFilial);
    }

    public function getOrderData($nrVendarest, $nrcomanda, $nrProdComVen, $isRodizio = false, $rodizioProds = null)
    {
        $data = [];
        $orders = $this->orderFactory->getOrderData($nrVendarest, $nrcomanda, $nrProdComVen);

        foreach ($orders as &$order) {
            $arrayItem = array(
                'NRVENDAREST'    => $order['NRVENDAREST'],
                'NRCOMANDA'      => $order['NRCOMANDA'],
                'CDPRODUTO'      => $order['CDPRODUTO'],
                'NMPRODUTO'      => $order['NMPRODUTO'],
                'QTPRODCOMVEN'   => $order['QTPRODCOMVEN'],
                'IDIMPRODUVEZ'   => $order['IDIMPRODUVEZ'],
                'IDPESAPROD'     => $order['IDPESAPROD'],
                'TXPRODCOMVEN'   => $order['TXPRODCOMVEN'],
                'CDPRODPROMOCAO' => $order['CDPRODPROMOCAO'],
                'IDOBSERVACAOAD' => $order['IDOBSERVACAOAD'],
                'NRSEQPRODCOM'   => $order['NRSEQPRODCOM'],
                'ITEMCOMBO'      => [],
            );

            if (!empty($order['CDPRODPROMOCAO'])) {
                foreach ($orders as &$itemCombo) {
                    if (((!empty($itemCombo['CDPRODPROMOCAO'])) && ($order['CDPRODUTO'] != $itemCombo['CDPRODUTO']) && ($order['CDPRODUTO'] == $itemCombo['CDPRODPROMOCAO']) && ($itemCombo['IDOBSERVACAOAD'] === 'N') && ($order['NRSEQPRODCOM'] == $itemCombo['NRSEQPRODCOM']))) {
                        $arrayItemCombo = array(
                            'NRVENDAREST'    => $itemCombo['NRVENDAREST'],
                            'NRCOMANDA'      => $itemCombo['NRCOMANDA'],
                            'CDPRODUTO'      => $itemCombo['CDPRODUTO'],
                            'NMPRODUTO'      => $itemCombo['NMPRODUTO'],
                            'QTPRODCOMVEN'   => $itemCombo['QTPRODCOMVEN'],
                            'IDIMPRODUVEZ'   => $itemCombo['IDIMPRODUVEZ'],
                            'IDPESAPROD'     => $itemCombo['IDPESAPROD'],
                            'TXPRODCOMVEN'   => $itemCombo['TXPRODCOMVEN'],
                            'CDPRODPROMOCAO' => $itemCombo['CDPRODPROMOCAO'],
                            'IDOBSERVACAOAD' => $itemCombo['IDOBSERVACAOAD'],
                            'NRSEQPRODCOM'   => $itemCombo['NRSEQPRODCOM'],
                            'ITEMCOMBO' => [],
                        );

                        array_push($arrayItem['ITEMCOMBO'], $arrayItemCombo);
                        $itemCombo['SON'] = true;
                    }
                }

                if(isset($itemCombo))
                    unset($itemCombo);
            }

            if (($isRodizio) && ($order['IDTIPOCOMPPROD'] == '6')) {
                if (empty($rodizioProds))
                    throw new \Exception('Erro ao montar impressão de produtos lançados no rodizio!');

                foreach ($rodizioProds as $itemRodizio) {
                    if (($itemRodizio['FATHERPROD'] == $order['CDPRODUTO'])) {
                        $arrayItemRodizio = array(
                            'NRVENDAREST'    => $itemRodizio['NRVENDAREST'],
                            'NRCOMANDA'      => $itemRodizio['NRCOMANDA'],
                            'CDPRODUTO'      => $itemRodizio['CDPRODUTO'],
                            'NMPRODUTO'      => $itemRodizio['NMPRODUTO'],
                            'QTPRODCOMVEN'   => $itemRodizio['QTPROCOMEST'],
                            'IDIMPRODUVEZ'   => 'N',
                            'IDPESAPROD'     => 'N',
                            'TXPRODCOMVEN'   => $itemRodizio['TXPRODCOMVENEST'],
                            'CDPRODPROMOCAO' => $order['CDPRODUTO'],
                            'IDOBSERVACAOAD' => 'N',
                            'NRSEQPRODCOM'   => $order['NRSEQPRODCOM'],
                            'ITEMCOMBO' => [],
                        );
    
                        array_push($arrayItem['ITEMCOMBO'], $arrayItemRodizio);
                    }
                }
                /*
                $comandaEst = $this->orderFactory->getComandaEst($order['CDFILIAL'], $order['NRVENDAREST'], $order['NRCOMANDA'], $order['NRPRODCOMVEN']);
                foreach ($comandaEst as $itemRodizio) {
                    $qtdLancamento = 0;
                    foreach ($rodizioProds as $lancProd) {
                        if($lancProd['NRVENDAREST'] === $itemRodizio['NRVENDAREST'] && $lancProd['NRPRODCOMVEN'] === $itemRodizio['NRPRODCOMVEN'] && $lancProd['CDPRODUTO'] === $itemRodizio['CDPRODUTO']) {
                            $qtdLancamento = $lancProd['QTPROCOMEST'];
                            $arrayItemRodizio = array(
                                'NRVENDAREST'    => $itemRodizio['NRVENDAREST'],
                                'NRCOMANDA'      => $itemRodizio['NRCOMANDA'],
                                'CDPRODUTO'      => $itemRodizio['CDPRODUTO'],
                                'NMPRODUTO'      => $itemRodizio['NMPRODUTO'],
                                'QTPRODCOMVEN'   => $qtdLancamento,
                                'IDIMPRODUVEZ'   => $itemRodizio['IDIMPRODUVEZ'],
                                'IDPESAPROD'     => $itemRodizio['IDPESAPROD'],
                                'TXPRODCOMVEN'   => $itemRodizio['TXPRODCOMVENEST'],
                                'CDPRODPROMOCAO' => $order['CDPRODUTO'],
                                'IDOBSERVACAOAD' => 'N',
                                'NRSEQPRODCOM'   => $order['NRSEQPRODCOM'],
                                'ITEMCOMBO' => [],
                            );
                            array_push($arrayItem['ITEMCOMBO'], $arrayItemRodizio);
                            break;
                        }                        
                    }
                }
                */
            }

            if (!isset($order['SON'])) {
                array_push($data, $arrayItem);
            }
        }

        if(isset($order))
            unset($order);

        return $data;
    }

    public function ValidateCards($generalParams, $comandas)
    {

        foreach ($comandas as $cmd) {
            $dscomanda = $this->util->toRight($cmd['code'], 10);
            $serviceTax = $this->util->toFloat($cmd['service_percentage']);
            $cmdDetails = $this->orderFactory->getComanda($dscomanda);

            if (!empty($cmdDetails)) {
                $vrcomisvend = $this->util->toFloat($cmdDetails['VRCOMISVENDE']);

                if ((!empty($serviceTax)) && ($serviceTax != $vrcomisvend)) {
                    $this->orderFactory->setServiceTax($cmdDetails, $serviceTax);
                } elseif ((!empty($vrcomisvend)) && ($serviceTax != $vrcomisvend)) {
                    $this->orderFactory->setServiceTax($cmdDetails, $serviceTax);
                }
            }
        }

        return null;
    }

    public function getCanceledProducts()
    {

        return $this->orderFactory->getCanceledProducts();
    }

    public function getUnsyncedProducts()
    {

        return $this->orderFactory->getUnsyncedProducts();
    }

    public function updateComandaVen($productDetails, $idproext)
    {

        return $this->orderFactory->updateComandaVen($productDetails, $idproext);
    }

    public function getSales()
    {
        return $this->orderFactory->getSales();
    }

    public function getTransferProduct()
    {
        return $this->orderFactory->getTransferProduct();
    }
}
